/**
 * Copyright (c) 2016-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.component.framework.junit.impl;

import javax.xml.namespace.QName;

import org.junit.Assert;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

public abstract class PCServiceConfiguration extends ServiceConfiguration {


    private final QName interfaceQName;

    private final QName serviceQName;

    /**
     * It is not final because if it is an autogenerate endpoint, it will be changed
     */
    private String endpointName;

    private Long timeout;

    public PCServiceConfiguration(final QName interfaceQName, final QName serviceQName, final String endpointName) {

        this.interfaceQName = interfaceQName;
        this.serviceQName = serviceQName;
        this.endpointName = endpointName;
    }

    /**
     * Meant to be called only by {@link ComponentInstanceImpl} to set the generated name of an autogenerate endpoint.
     * 
     * Hence it has package visibility.
     */
    void setEndpointName(String endpointName) {
        Assert.assertEquals(this.endpointName, "autogenerate");
        this.endpointName = endpointName;
    }

    public QName getInterfaceName() {
        return interfaceQName;
    }

    public QName getServiceName() {
        return serviceQName;
    }

    public final String getEndpointName() {
        return endpointName;
    }

    public Long getTimeout() {
        return timeout;
    }

    /**
     * 
     * @param timeout
     *            CDK timeout
     */
    public void setTimeout(final long timeout) {
        this.timeout = timeout;
    }

    @Override
    public String toString() {
        return "ServiceConfiguration [interfaceQName=" + interfaceQName
                + ", serviceQName=" + serviceQName + ", endpointName=" + endpointName + "]";
    }

    /**
     * Override to add extra custom configuration on the JBI service element (such as adding elements more complex than
     * simple parameters...).
     */
    protected void extraServiceConfiguration(final Document jbiDocument, final Element service) {
        // nothing
    }

}
