/**
 * Copyright (c) 2016-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.component.framework.junit.impl.mock;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import javax.xml.namespace.QName;

import org.ow2.petals.jbi.servicedesc.endpoint.util.ServiceEndpointHelper;

public class MockEndpointDirectory {

    private final Set<MockServiceEndpoint> serviceEndpoints = new HashSet<>();

    public void activateEndpoint(final MockServiceEndpoint serviceEndpoint) {
        serviceEndpoints.add(serviceEndpoint);
    }

    public void deactivateEndpoint(final MockServiceEndpoint serviceEndpoint) {
        serviceEndpoints.remove(serviceEndpoint);
    }

    public void clearEndpoints() {
        serviceEndpoints.clear();
    }

    public MockServiceEndpoint resolveEndpoint(final QName service, final String name) {
        MockServiceEndpoint result = null;
        for (final MockServiceEndpoint serviceEndpoint : serviceEndpoints) {
            if (name.equals(serviceEndpoint.getEndpointName()) && service.equals(serviceEndpoint.getServiceName())) {
                result = serviceEndpoint;
                break;
            }
        }

        return result;
    }

    public Collection<MockServiceEndpoint> resolveEndpointsForService(final QName service) {
        final Collection<MockServiceEndpoint> endpoints = new ArrayList<>();
        for (final MockServiceEndpoint serviceEndpoint : serviceEndpoints) {
            if (service.equals(serviceEndpoint.getServiceName())) {
                endpoints.add(serviceEndpoint);
            }
        }

        return endpoints;
    }

    public Collection<MockServiceEndpoint> resolveEndpoints(final QName interfaceName) {

        if (interfaceName == null) {
            return new ArrayList<>(serviceEndpoints);
        }

        final Collection<MockServiceEndpoint> endpoints = new ArrayList<>();

        for (final MockServiceEndpoint serviceEndpoint : serviceEndpoints) {
            if (ServiceEndpointHelper.matches(serviceEndpoint, interfaceName)) {
                endpoints.add(serviceEndpoint);
            }
        }

        return endpoints;
    }
}
