/**
 * Copyright (c) 2017-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.component.framework.junit.mbean;

import static org.junit.Assert.assertNotNull;

import java.io.InputStream;
import java.lang.reflect.Field;
import java.util.logging.Logger;

import javax.management.MalformedObjectNameException;

import org.junit.Rule;
import org.junit.rules.RuleChain;
import org.junit.rules.TestRule;
import org.ow2.petals.component.framework.DefaultBootstrap;
import org.ow2.petals.component.framework.JBIBootstrap;
import org.ow2.petals.component.framework.jbidescriptor.CDKJBIDescriptorBuilder;
import org.ow2.petals.component.framework.jbidescriptor.generated.Jbi;
import org.ow2.petals.jbi.descriptor.JBIDescriptorException;

/**
 * Abstract class to test component bootstrap extending {@link DefaultBootstrap}.
 * 
 * @author Christophe DENEUX - Linagora
 *
 */
public abstract class AbstractBootstrapTest {

    protected final EmbeddedJmxServerConnector embeddedJmxSrvCon = new EmbeddedJmxServerConnector();

    protected final EmbeddedJmxClient jmxClient = new EmbeddedJmxClient(this.embeddedJmxSrvCon);

    @Rule
    public TestRule jmxTestRuleChain = RuleChain.outerRule(this.embeddedJmxSrvCon).around(this.jmxClient);

    protected AbstractBootstrapTest() throws MalformedObjectNameException {
        // Constructor requires to declare the exception thrown by 'jmxClient'
    }

    /**
     * Initialize the given component bootstrap from the JBI descriptor of the component.
     * 
     * @param bootstrap
     *            The component bootstrap to initialize
     * @return the given component bootstrap
     */
    protected DefaultBootstrap initBootstrap(final DefaultBootstrap bootstrap) throws IllegalArgumentException,
            IllegalAccessException, SecurityException, NoSuchFieldException, JBIDescriptorException {

        final InputStream defaultJbiDescriptorStream = Thread.currentThread().getContextClassLoader()
                .getResourceAsStream("jbi/jbi.xml");
        assertNotNull("The component JBI descriptor is missing", defaultJbiDescriptorStream);
        final Jbi jbiComponentConfiguration = CDKJBIDescriptorBuilder.getInstance()
                .buildJavaJBIDescriptor(defaultJbiDescriptorStream);

        return this.initBootstrap(bootstrap, jbiComponentConfiguration);
    }

    /**
     * Initialize the given component bootstrap from the given JBI descriptor.
     * 
     * @param bootstrap
     *            The component bootstrap to initialize
     * @param jbiComponentConfiguration
     *            The JBI descriptor to use
     * @return the given component bootstrap
     */
    protected DefaultBootstrap initBootstrap(final DefaultBootstrap bootstrap, final Jbi jbiComponentConfiguration)
            throws IllegalArgumentException, IllegalAccessException, NoSuchFieldException, SecurityException {

        final Field jbiComponentCfgField = JBIBootstrap.class.getDeclaredField("jbiComponentConfiguration");
        jbiComponentCfgField.setAccessible(true);
        jbiComponentCfgField.set(bootstrap, jbiComponentConfiguration);
        final Field loggerField = JBIBootstrap.class.getDeclaredField("logger");
        loggerField.setAccessible(true);
        loggerField.set(bootstrap, Logger.getLogger(this.getClass().getName()));

        return bootstrap;
    }

}
