/**
 * Copyright (c) 2011-2012 EBM WebSourcing, 2012-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.component.framework.junit.rule;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;

import javax.xml.namespace.QName;

import org.junit.Rule;
import org.junit.Test;
import org.ow2.petals.component.framework.junit.impl.ComponentConfiguration;
import org.ow2.petals.component.framework.junit.impl.ProvidesServiceConfiguration;
import org.ow2.petals.component.framework.junit.impl.ServiceConfiguration;
import org.ow2.petals.junit.rules.log.handler.InMemoryLogHandler;

/**
 * @author Marc Jambert - EBM WebSourcing
 * @author Christophe DENEUX - Linagora
 */
public class ComponentUnderTestTest {

    private static final String SVC_CFG_ID = "svc-cfg-id";

    @Rule
    public final InMemoryLogHandler inMemoryLogHandler = new InMemoryLogHandler();
    
    @Rule
    public final ComponentUnderTest componentUnderTest = new ComponentUnderTest(new ComponentConfiguration(
            "componentUnderTest"), true, false);

    @Test
    public void workWithComponentUnderTest() throws Exception {

        // Check the initial state of the component under test: STOPPED
        assertTrue("Component under test not installed", componentUnderTest.isInstalled());
        assertFalse("Unexpected initial component state", componentUnderTest.isStarted());

        // Check to start the component under test
        componentUnderTest.start();
        assertTrue("Component under test not started", componentUnderTest.isStarted());

        // Check to stop the component under test
        componentUnderTest.stop();
        assertFalse("Component under test not stopped", componentUnderTest.isStarted());

        // Check to deploy/undeploy a service provider
        componentUnderTest.start();
        assertTrue("Component under test not started", componentUnderTest.isStarted());
        final ServiceConfiguration providerConfiguration = this.createTestProvideServiceBaseConfiguration();
        assertFalse("Service provider alredy installed", componentUnderTest.isServiceDeployed(SVC_CFG_ID));
        componentUnderTest.deployService(SVC_CFG_ID, new ServiceConfigurationFactory() {

            @Override
            public ServiceConfiguration create() {
                return providerConfiguration;
            }
        });
        assertTrue("Service provider not installed", componentUnderTest.isServiceDeployed(SVC_CFG_ID));
        assertNotNull(componentUnderTest.getServiceConfiguration(SVC_CFG_ID));
        componentUnderTest.undeployService(SVC_CFG_ID);
        assertFalse("Service provider still installed", componentUnderTest.isServiceDeployed(SVC_CFG_ID));
        assertNull(componentUnderTest.getServiceConfiguration(SVC_CFG_ID));

        componentUnderTest.deployService(SVC_CFG_ID, new ServiceConfigurationFactory() {

            @Override
            public ServiceConfiguration create() {
                return providerConfiguration;
            }
        });
        assertTrue("Service provider not installed", componentUnderTest.isServiceDeployed(SVC_CFG_ID));

        // Check to undeploy all services
        componentUnderTest.undeployAllServices();
        assertFalse("A service provider is still installed", componentUnderTest.isServiceDeployed(SVC_CFG_ID));

        componentUnderTest.shutdown();
        assertFalse("Component under test not installed", componentUnderTest.isInstalled());
    }
    
    private ServiceConfiguration createTestProvideServiceBaseConfiguration() throws Exception {
        final ServiceConfiguration testInternalServiceConguration = new ProvidesServiceConfiguration(
                new QName("http://testProvideService", "testProvideInterfaceName"),
                new QName("http://testProvideService", "testServiceName"), "testEndpointName");

        return testInternalServiceConguration;
    }
}
